{{
Project Lynxmotion Phoenix
Description: Phoenix, control file.
  The control input subroutine for the phoenix software is placed in this file.
  Can be used with V2.0 and above
Configuration version: V1.0
Hardware setup: PS2

Date: Nov 17, 2010

       Walk method 1:
       - Left Stick    Walk/Strafe
       - Right Stick   Rotate

       Walk method 2:
       - Left Stick    Disable
       - Right Stick   Walk/Rotate


PS2 CONTROLS:
       [Common Controls]
       - Start         Turn on/off the bot
       - L1            Toggle Shift mode
       - L2            Toggle Rotate mode
       - Circle        Toggle Single leg mode
       - Cross         Toggle GP Mode
       - Square        Toggle Balance mode
       - Triangle              Move body to 35 mm from the ground (walk pos) 
                                       and back to the ground
       - D-Pad up              Body up 10 mm
       - D-Pad down    Body down 10 mm
       - D-Pad left    decrease speed with 50mS
       - D-Pad right   increase speed with 50mS

       [Walk Controls]
       - select                Switch gaits
       - Left Stick    (Walk mode 1) Walk/Strafe
                                       (Walk mode 2) Disable
       - Right Stick   (Walk mode 1) Rotate,           
                                       (Walk mode 2) Walk/Rotate
       - R1                    Toggle Double gait travel speed
       - R2                    Toggle Double gait travel length

       [Shift Controls]
       - Left Stick    Shift body X/Z
       - Right Stick   Shift body Y and rotate body Y

       [Rotate Controls]
       - Left Stick    Rotate body X/Z
       - Right Stick   Rotate body Y   

       [Single leg Controls]
       - select                Switch legs
       - Left Stick    Move Leg X/Z (relative)
       - Right Stick   Move Leg Y (absolute)
       - R2                    Hold/release leg position

       [GP Player Controls]
       - select                Switch Sequences
       - R2                    Start Sequence


NEW IN V1.0
 - First Release


}}

OBJ
  PS2   : "PS2_controller_new"       
  DBG   : "SerialMirror"
  CFG   : "Phoenix_Config"

VAR
'=============================================================================
' Global - Local to this file only...
'=============================================================================
  long g_palCData

' Local settings that are not passed back...
  byte DoubleHeightOn
  byte DoubleTravelOn
  byte WalkMethod
  
' GP Player settings... Will need info as well from main object...
  byte g_fGPEnable      ' need to set as part of the init...

' Not sure about these...
  word    _BodyYOffset
  byte    _BodyYShift 

' Non exported values...

  byte    bControlMode       ' Our current control mode...  


con
  WALKMODE      =    0
  TRANSLATEMODE =    1
  ROTATEMODE    =    2
  SINGLELEGMODE =    3
  GPPLAYERMODE  =    4

  GPSTARTED     = 0    ' Used in the GPSetState function...
  GPCOMPLETED   = 1
  GPERROR       = 2

  TravelDeadZone = 4  'The deadzone for the analog input from the remote
  cTravelDeadZone =  4          'The deadzone for the analog input from the remote

{{ The following functions are the standard methods that is called by the main phoenix code.
All Control classes need to implement these...
}}


PUB Init(palCData, GPEnable)
'' This is the main Init function that is called by the Phoenix code to initialize
'' the controller. We are passed in information about if this phoenix support
'' General purpose sequences.
  g_palCData := palCData
  g_fGPEnable := GPEnable


  LONG[g_palCData][CFG#ISI_HexOn] := 0                     ' Assume the Hex is logically off to start off with...
  LONG[g_palCData][CFG#ISI_GaitType] := 0                   ' Start off assuming Gait 0...

  DBG.Str(string("PS2 Init",13))
  PS2.Start(CFG#PS2_FIRSTPIN, 20_000) 'first_pin 0, Poll every 5ms  (1ms works for MadCatz wired but not for Predator wireless)


  'Body Positions
  LONG[g_palCData][CFG#ISI_BodyPosX] := 0
  LONG[g_palCData][CFG#ISI_BodyPosY] := 0
  LONG[g_palCData][CFG#ISI_BodyPosZ] := 0
      
  'Body Rotations
  LONG[g_palCData][CFG#ISI_BodyRotX1] := 0
  LONG[g_palCData][CFG#ISI_BodyRotY1] := 0
  LONG[g_palCData][CFG#ISI_BodyRotZ1] := 0

   
  _BodyYOffset := 0
  _BodyYShift := 0
  LONG[g_palCData][CFG#ISI_LegLiftHeight] := 50
  LONG[g_palCData][CFG#ISI_BalanceMode] := 0
  LONG[g_palCData][CFG#ISI_SelectedLeg] := 255 ' No Leg selected
  bControlMode := WALKMODE
   
  DoubleHeightOn := false
  DoubleTravelOn := false


  return true 

PUB ClearTravelData
    LONG[g_palCData][CFG#ISI_TravelLengthX]:=0
    LONG[g_palCData][CFG#ISI_TravelLengthZ]:=0
    LONG[g_palCData][CFG#ISI_TravelRotationY]:=0
 


PUB Input | wGPSeqPtr, w

 
  ' Then try to receive a packet of information from the PS2.
  if PS2.read_gamepad

    ' Lets see if the user pressed the start button. 
    if PS2.ButtonPressed(PS2#PSB_START)
      if(LONG[g_palCData][CFG#ISI_HexOn]) 
                                'Turn off
         LONG[g_palCData][CFG#ISI_BodyPosX] := 0
         LONG[g_palCData][CFG#ISI_BodyPosY] := 0
         LONG[g_palCData][CFG#ISI_BodyPosZ] := 0
         LONG[g_palCData][CFG#ISI_BodyRotX1] := 0
         LONG[g_palCData][CFG#ISI_BodyRotY1] := 0
         LONG[g_palCData][CFG#ISI_BodyRotZ1] := 0
         LONG[g_palCData][CFG#ISI_TravelLengthX] := 0
         LONG[g_palCData][CFG#ISI_TravelLengthZ] := 0
         LONG[g_palCData][CFG#ISI_TravelRotationY] := 0
         _BodyYOffset := 0
         _BodyYShift := 0
         LONG[g_palCData][CFG#ISI_SelectedLeg] := 255
                      
         DBG.str(string("Hex Off",13))
'         PS2.PlaySounds(@SND_HEXOFF)
         CFG.MSysSounds (CFG#SSND_HEXOFF)
         LONG[g_palCData][CFG#ISI_HexOn] := 0
      else 
        'Turn on
         DBG.str(string("Hex On - PS2",13))
'          PS2.PlaySounds(CFG#SSND_HEXON)
         CFG.MSysSounds (CFG#SSND_HEXON)
         LONG[g_palCData][CFG#ISI_HexOn] := 1

    if LONG[g_palCData][CFG#ISI_HexOn]
      ' Translate mode
      if PS2.ButtonPressed(PS2#PSB_L1) ' check L1 button
        if bControlMode <> TRANSLATEMODE
           bControlMode := TRANSLATEMODE
           CFG.MSysSounds (CFG#SSND_BEEP)
           DBG.str(string("Translate",13))
        elseif LONG[g_palCData][CFG#ISI_SelectedLeg] == 255
           bControlMode := WALKMODE
           CFG.MSysSounds (CFG#SSND_BEEP2)
           DBG.str(string("Walk",13))
        else
           bControlMode := SINGLELEGMODE        

      ' Rotate mode     
      if PS2.ButtonPressed(PS2#PSB_L2) ' L2 button
         if bControlMode <> ROTATEMODE
           DBG.str(string("Rotate",13))
           CFG.MSysSounds (CFG#SSND_BEEP)
           bControlMode := ROTATEMODE
         elseif LONG[g_palCData][CFG#ISI_SelectedLeg] == 255
           DBG.str(string("Walk",13))
           CFG.MSysSounds (CFG#SSND_BEEP2)
           bControlMode := WALKMODE
         else
           bControlMode := SINGLELEGMODE        

      ' Single Leg Mode   
      if PS2.ButtonPressed(PS2#PSB_CIRCLE) ' Circle button test
         CFG.MSysSounds (CFG#SSND_BEEP)
         if bControlMode<>SINGLELEGMODE 
             DBG.str(string("Single Leg",13))
             bControlMode := SINGLELEGMODE        
             if (LONG[g_palCData][CFG#ISI_SelectedLeg]==255) ' none
                LONG[g_palCData][CFG#ISI_SelectedLeg] := 0                            ' could choose later how to get the right leg
         else 
             bControlMode := WALKMODE
             CFG.MSysSounds (CFG#SSND_BEEP2)
             LONG[g_palCData][CFG#ISI_SelectedLeg] := 255  'none

      ' GP Player
      if PS2.ButtonPressed(PS2#PSB_CROSS) ' Cross button
      
        if (g_fGPEnable)    'F Button GP Player Mode Mode on/off -- SSC supports this mode
            if bControlMode <> GPPLAYERMODE
               LONG[g_palCData][CFG#ISI_BodyPosX] := 0
               LONG[g_palCData][CFG#ISI_BodyPosZ] := 0
               LONG[g_palCData][CFG#ISI_BodyRotX1] := 0
               LONG[g_palCData][CFG#ISI_BodyRotY1] := 0
               LONG[g_palCData][CFG#ISI_BodyRotZ1] := 0
               LONG[g_palCData][CFG#ISI_TravelLengthX] := 0
               LONG[g_palCData][CFG#ISI_TravelLengthZ] := 0
               LONG[g_palCData][CFG#ISI_TravelRotationY] := 0
                     
               LONG[g_palCData][CFG#ISI_SelectedLeg] := 255  'none
               LONG[g_palCData][CFG#ISI_fSLHold] := 0
                   
               CFG.MSysSounds (CFG#SSND_BEEP)
               bControlMode := GPPLAYERMODE
            else
               CFG.MSysSounds (CFG#SSND_BEEP2)
               bControlMode := WALKMODE
        else
            CFG.MSysSounds (CFG#SSND_BEEP2)

      '
      ' Common settings      
      if PS2.ButtonPressed(PS2#PSB_SQUARE) 'Square - Balance mode
        if not LONG[g_palCData][CFG#ISI_BalanceMode] 
           CFG.MSysSounds (CFG#SSND_BEEP)
          LONG[g_palCData][CFG#ISI_BalanceMode] := 1
        else
          CFG.MSysSounds (CFG#SSND_BEEP2)
         LONG[g_palCData][CFG#ISI_BalanceMode] := 0
    
      if PS2.ButtonPressed(PS2#PSB_TRIANGLE)  ' Stand up or sit down...
        if _BodyYOffset > 0
            _BodyYOffset := 0
        else
            _BodyYOffset := 35

      if PS2.ButtonPressed(PS2#PSB_PAD_UP)  ' ..
         CFG.MSysSounds (CFG#SSND_BEEP)
          _BodyYOffset += 10
                
      if PS2.ButtonPressed(PS2#PSB_PAD_DOWN)  ' ..
         CFG.MSysSounds (CFG#SSND_BEEP)
          _BodyYOffset -= 10
        
      if PS2.ButtonPressed(PS2#PSB_PAD_RIGHT)  ' ..
          IF LONG[g_palCData][CFG#ISI_SpeedControl]>0
             CFG.MSysSounds (CFG#SSND_BEEP)
            LONG[g_palCData][CFG#ISI_SpeedControl] -=  50
          else
             CFG.MSysSounds (CFG#SSND_BEEP2)

        
      if PS2.ButtonPressed(PS2#PSB_PAD_LEFT)  ' ..
          IF LONG[g_palCData][CFG#ISI_SpeedControl]<2000
             CFG.MSysSounds (CFG#SSND_BEEP)
            LONG[g_palCData][CFG#ISI_SpeedControl] += 50
          else
             CFG.MSysSounds (CFG#SSND_BEEP2)

        
      if bControlMode == WALKMODE 
          if PS2.ButtonPressed(PS2#PSB_SELECT) and ||(LONG[g_palCData][CFG#ISI_TravelLengthX])<cTravelDeadZone and  ||(LONG[g_palCData][CFG#ISI_TravelLengthZ])<cTravelDeadZone and ||(LONG[g_palCData][CFG#ISI_TravelRotationY]*2)<cTravelDeadZone
              'Switch gait
              if LONG[g_palCData][CFG#ISI_GaitType] < 7
                CFG.MSysSounds (CFG#SSND_BEEP)
                LONG[g_palCData][CFG#ISI_GaitType]++
              else
                 CFG.MSysSounds (CFG#SSND_BEEP2)
                  LONG[g_palCData][CFG#ISI_GaitType] := 0
                  
          if PS2.ButtonPressed(PS2#PSB_R1)
              DoubleHeightOn := not DoubleHeightOn
              IF DoubleHeightOn
                  LONG[g_palCData][CFG#ISI_LegLiftHeight] := 80          
              ELSE                  
                  LONG[g_palCData][CFG#ISI_LegLiftHeight] := 50

          if PS2.ButtonPressed(PS2#PSB_R2)
              DoubleTravelOn := not DoubleHeightOn

          if PS2.ButtonPressed(PS2#PSB_R3)
              WalkMethod := not WalkMethod

          if WalkMethod
              LONG[g_palCData][CFG#ISI_TravelLengthZ] := PS2.Analog(PS2#PSS_RY) - 128 'Right Stick Up/Down     
          else    
              LONG[g_palCData][CFG#ISI_TravelLengthX] := -(PS2.Analog(PS2#PSS_LX) - 128)
              LONG[g_palCData][CFG#ISI_TravelLengthZ] := (PS2.Analog(PS2#PSS_LY) - 128)

          IF not DoubleTravelOn
              LONG[g_palCData][CFG#ISI_TravelLengthX] := LONG[g_palCData][CFG#ISI_TravelLengthX]/2
              LONG[g_palCData][CFG#ISI_TravelLengthZ] := LONG[g_palCData][CFG#ISI_TravelLengthZ]/2
                
          LONG[g_palCData][CFG#ISI_TravelRotationY] := -(PS2.Analog(PS2#PSS_RX) - 128)/4  ';Right Stick Left/Right  
    
      elseif bControlMode == TRANSLATEMODE
          LONG[g_palCData][CFG#ISI_BodyPosX] := (PS2.Analog(PS2#PSS_LX) - 128)/2
          LONG[g_palCData][CFG#ISI_BodyPosZ] := -(PS2.Analog(PS2#PSS_LY) - 128)/2
          LONG[g_palCData][CFG#ISI_BodyRotY1] := (PS2.Analog(PS2#PSS_RX) - 128)*2
          _BodyYShift := (-(PS2.Analog(PS2#PSS_RY) - 128)/2)

      elseif bControlMode == ROTATEMODE
          LONG[g_palCData][CFG#ISI_BodyRotX1] := (PS2.Analog(PS2#PSS_LY) - 128)*2
          LONG[g_palCData][CFG#ISI_BodyRotY1] := (PS2.Analog(PS2#PSS_RX) - 128)*2
          LONG[g_palCData][CFG#ISI_BodyRotZ1] := (PS2.Analog(PS2#PSS_LX) - 128)*2
          _BodyYShift := (-(PS2.Analog(PS2#PSS_RY) - 128)/2)

          
       'Switch single leg
      elseif (bControlMode==SINGLELEGMODE) 
          if PS2.ButtonPressed(PS2#PSB_SELECT) 
              'Change Legs
              if LONG[g_palCData][CFG#ISI_SelectedLeg] < 5
                LONG[g_palCData][CFG#ISI_SelectedLeg]++
              else
                  LONG[g_palCData][CFG#ISI_SelectedLeg]  := 0

          'Single Leg Mode
          LONG[g_palCData][CFG#ISI_SLLegX] := ((PS2.Analog(PS2#PSS_LX)-128)/2) 
          LONG[g_palCData][CFG#ISI_SLLegY] := (PS2.Analog(PS2#PSS_RY)-128)/10
          LONG[g_palCData][CFG#ISI_SLLegZ] := (PS2.Analog(PS2#PSS_LY)-128)/2

          if PS2.ButtonPressed(PS2#PSB_R2)
              LONG[g_palCData][CFG#ISI_fSLHold] := not LONG[g_palCData][CFG#ISI_fSLHold]
      elseif (bControlMode == GPPLAYERMODE)
          if PS2.ButtonPressed(PS2#PSB_SELECT) 
              if (LONG[g_palCData][CFG#ISI_GPStart] == 0)
                  if LONG[g_palCData][CFG#ISI_GPSeq] < 5
                      LONG[g_palCData][CFG#ISI_GPSeq]++
                  else
                      LONG[g_palCData][CFG#ISI_GPSeq] := 0
          if PS2.ButtonPressed(PS2#PSB_R2)
              LONG[g_palCData][CFG#ISI_GPStart] := 1

    'Calculate walking time delay
    LONG[g_palCData][CFG#ISI_InputTimeDelay] := 128 -   (( ||(PS2.Analog(PS2#PSS_LX)-128) #> ||(PS2.Analog(PS2#PSS_LY)-128)) #>  ||(PS2.Analog(PS2#PSS_RX)-128))

  LONG[g_palCData][CFG#ISI_BodyPosY] := (_BodyYOffset + _BodyYShift) #> 0



return

pub GPSetState (state)
    case state
        GPERROR:
           CFG.MSysSounds (CFG#SSND_BEEP2)
          LONG[g_palCData][CFG#ISI_GPStart] := 0         ' clear out the state  
        GPSTARTED:
'          PS2.OutputString(STRING("Start Sequence"))
        GPCOMPLETED:
'          PS2.OutputString(STRING("Completed"))
          LONG[g_palCData][CFG#ISI_GPStart] := 0         ' clear out the state  
           