{{
Project Lynxmotion Phoenix
Description: Phoenix, control file.
  The control input subroutine for the phoenix software is placed in this file.
  Can be used with V2.0 and above
Configuration version: V1.0
Hardware setup: DIY RC receiver.


Date: Nov 17, 2010

NEW IN V1.0
 - First Release

RC Controller - Will depend on what capabilities different RC receivers use.


}}

OBJ
  RCV   : "ServoInput"          ' This is the simple function that will keep looking at the RC Pulses       
  DBG   : "SerialMirror"
  CFG   : "Phoenix_Config"                                     ' change this to the one specific for your setup.

con
  WALKMODE      =    0
  TRANSLATEMODE =    1
  ROTATEMODE    =    2
  SINGLELEGMODE =    3
  GPPLAYERMODE  =    4

  GPSTARTED     = 0    ' Used in the GPSetState function...
  GPCOMPLETED   = 1
  GPERROR       = 2

  PWAI_NUMPINS  = 6
  PWAI_RLR      = 0
  PWAI_RUD      = 1
  PWAI_LUD      = 2
  PWAI_LLR      = 3
  PWAI_GEAR     = 4             ' Geaer pin 2 settings on Hitec 6
  PWAI_AUX      = 5             ' Aux switch 

  MAXPW         = 2000
  MINPW         = 1000
  TravelDeadZone = 10  'The deadzone for the analog input from the remote


  MAX_RC_FAILURE = 8

VAR
'=============================================================================
' Global - Local to this file only...
'=============================================================================
  long g_palCData
  byte _fGPEnable      ' need to set as part of the init...

' Non exported values...

  byte    bControlMode       ' Our current control mode...  
  byte   _fSwitchOnPrev
  byte   _fSwitchOn
  byte   _fInitOffsets           ' Have we init the centers yet?
  byte   _cRCFailure             ' how many times in a row have we failed?

' Normalized RC values
  long  alNPW[PWAI_NUMPINS]
  long  alPWOffsets[PWAI_NUMPINS]


DAT
        ' Wish there was a way to define an array here using a CON value for size...
        rcPins          LONG CFG#RC_PIN0, CFG#RC_PIN1, CFG#RC_PIN2, CFG#RC_PIN3, CFG#RC_PIN4, CFG#RC_PIN5
        alPW            LONG 1, 1, 1, 1, 1, 1

PUB Init(palCData, GPEnable)
  g_palCData := palCData                                ' Remember pointer to control data
  _fGPEnable := GPEnable                                ' Likewise for if GP is enabled or not
  
  LONG[g_palCData][CFG#ISI_HexOn] := 0                     ' Assume the Hex is logically off to start off with...
  LONG[g_palCData][CFG#ISI_GaitType] := 0                   ' Start off assuming Gait 0...

  DBG.Str(string("RC Controller Init",13))
  ' BUGBUG: should somehow move some of these defines to config file...
  RCV.start(@rcPins,6,@alPW)


  'Body Positions
  LONG[g_palCData][CFG#ISI_BodyPosX] := 0
  LONG[g_palCData][CFG#ISI_BodyPosY] := 0
  LONG[g_palCData][CFG#ISI_BodyPosZ] := 0
      
  'Body Rotations
  LONG[g_palCData][CFG#ISI_BodyRotX1] := 0
  LONG[g_palCData][CFG#ISI_BodyRotY1] := 0
  LONG[g_palCData][CFG#ISI_BodyRotZ1] := 0

   
  LONG[g_palCData][CFG#ISI_LegLiftHeight] := 50
  LONG[g_palCData][CFG#ISI_BalanceMode] := 0
  LONG[g_palCData][CFG#ISI_SelectedLeg] := 255 ' No Leg selected
  bControlMode := WALKMODE
  _cRCFailure := 0
  _fInitOffsets := 0
   
  return true 

PUB ClearTravelData
    LONG[g_palCData][CFG#ISI_TravelLengthX]:=0
    LONG[g_palCData][CFG#ISI_TravelLengthZ]:=0
    LONG[g_palCData][CFG#ISI_TravelRotationY]:=0
 


PUB Input | i, fRCValid 
 
  ' The ServoInput code runs in the background simply runs in the background and fils in an array.  We will first
  ' validate that the data is valid before we use it.  If not we will assume that we lost contact with the controller and turn
  ' the hex logically off.
  fRCValid := true              ' start off assuming that we are receiving valid RC Data.
  repeat i from 0 to PWAI_NUMPINS-1
     if alPW[i] =< MINPW or alPW[i] => MAXPW
        fRCValid := false
     else
        alNPW[i] := alPW[i] + alPWOffsets[i]
        if (i == PWAI_RLR) or (i == PWAI_RUD) or (i == PWAI_LLR)
            if ||(alNPW[i]-1500) < TravelDeadZone
                alNPW[i] := 1500 


  
  if fRCValid
      _cRCFailure := 0
      if not LONG[g_palCData][CFG#ISI_HexOn]            ' Let the user know if this is our first valid data.
         ' See if we have initialized the Offsets yet....
         if  _fInitOffsets < 8
             repeat i from 0 to PWAI_NUMPINS-1
                 alPWOffsets[i] += alPW[i]
             _fInitOffsets++
             if _fInitOffsets == 8
                repeat i from 0 to PWAI_NUMPINS-1 
                   if (i == PWAI_RLR) or (i == PWAI_RUD) or (i == PWAI_LLR)
                       alPWOffsets[i] := 1500 - (alPWOffsets[i]>>3)
                   else  
                       alPWOffsets[i] := 0
             return  ' don't process until we have a complete set...          
                 
        'Turn on
         DBG.str(string("Hex On",13))
         CFG.MSysSounds (CFG#SSND_HEXON)
         LONG[g_palCData][CFG#ISI_HexOn] := 1

  ' Lets see what state we should be in.
      if alNPW[PWAI_AUX] <  1100 
          if bControlMode <> WalkMode 
              CFG.MSysSounds(CFG#SSND_BEEP)
              DBG.Str(string("Walk Mode",13))
              bControlMode := WalkMode
      elseif alNPW[PWAI_AUX] < 1300
          if bControlMode <> TranslateMode
              DBG.Str(string("Translate Mode",13))
              CFG.MSysSounds(CFG#SSND_BEEP)
              bControlMode := TranslateMode
      elseif alNPW[PWAI_AUX] < 1500      
          if bControlMode <> RotateMode
              CFG.MSysSounds(CFG#SSND_BEEP)
              DBG.Str(string("Rotate Mode",13))
              bControlMode := RotateMode
      elseif alNPW[PWAI_AUX] < 1700      
          if bControlMode <> SingleLegMode
              DBG.Str(string("Single Leg Mode",13))
              bControlMode := SingleLegMode        
              CFG.MSysSounds(CFG#SSND_BEEP)
              LONG[g_palCData][CFG#ISI_SelectedLeg] := 0
      else
          if bControlMode <> GPPLAYERMODE
              DBG.Str(string("GP Mode",13))
              bControlMode := GPPLAYERMODE        
              CFG.MSysSounds(CFG#SSND_BEEP)
      
          ' Note Have not done anything here yet for running sequences...
          ' likewise the ability to turn balance on or off...


      _fSwitchOnPrev := _fSwitchOn
      _fSwitchOn := (alNPW[PWAI_GEAR] > 1500)

      if _fSwitchOn <> _fSwitchOnPrev                   ' the switch toggled.
          if bControlMode == WalkMode                   '    '1-8 Button Gait select   
              if LONG[g_palCData][CFG#ISI_GaitType] < 7
                  LONG[g_palCData][CFG#ISI_GaitType] +=  1 
                  CFG.MSysSounds(CFG#SSND_BEEP)
              else
                  LONG[g_palCData][CFG#ISI_GaitType] := 0
                  CFG.MSysSounds(CFG#SSND_BEEP2)
          elseif bControlMode == SinglelegMode          ' use the toggle to select which leg to move
              if ~LONG[g_palCData][CFG#ISI_SelectedLeg] < 5
                  LONG[g_palCData][CFG#ISI_SelectedLeg] :=  ~LONG[g_palCData][CFG#ISI_SelectedLeg] + 1
                  CFG.MSysSounds(CFG#SSND_BEEP)
              else
                  LONG[g_palCData][CFG#ISI_SelectedLeg] := 255
                  CFG.MSysSounds(CFG#SSND_BEEP2)
                  
          elseif bControlMode == GPPLAYERMODE
              if (LONG[g_palCData][CFG#ISI_GPStart] == 0)
                  if LONG[g_palCData][CFG#ISI_GPSeq] < 5
                      LONG[g_palCData][CFG#ISI_GPSeq]++
                  else
                      LONG[g_palCData][CFG#ISI_GPSeq] := 0
                        
      'Body Height
      'BodyPosY = bPacket(PKT_LJOYUD) / 2
      LONG[g_palCData][CFG#ISI_BodyPosY] := ((alNPW[PWAI_LUD] #> 1100) - 1100) / 6
       
      'Leg lift height - Right slider has value 0-255 translate to 30-93
      'LONG[g_palCData][CFG#ISI_LegLiftHeight] = 30 + bPacket(PKT_RSLIDER)/5
      LONG[g_palCData][CFG#ISI_LegLiftHeight] := 60                              ' no control of this yet...
       
      'Walk mode                      
      if bControlMode == WalkMode                  
          LONG[g_palCData][CFG#ISI_TravelLengthX] := -(alNPW[PWAI_RLR] - 1500) / 3
          LONG[g_palCData][CFG#ISI_TravelLengthZ] := -(alNPW[PWAI_RUD] - 1500) / 3   
          LONG[g_palCData][CFG#ISI_TravelRotationY] := -(alNPW[PWAI_LLR] - 1500)/10
       
      'Body move      
      elseif bControlMode == TranslateMode     
          LONG[g_palCData][CFG#ISI_BodyPosX] := (alNPW[PWAI_RLR]-1500)/6
          LONG[g_palCData][CFG#ISI_BodyPosZ] := (alNPW[PWAI_RUD]-1500)/6
          LONG[g_palCData][CFG#ISI_BodyRotY1] := (alNPW[PWAI_LLR]-1500)/2
       
      'Body rotate    
      elseif bControlMode == RotateMode        
          LONG[g_palCData][CFG#ISI_BodyRotX1] := (alNPW[PWAI_RUD]-1500)/2
          LONG[g_palCData][CFG#ISI_BodyRotY1] := (alNPW[PWAI_LLR]-1500)/2
          LONG[g_palCData][CFG#ISI_BodyRotZ1] := -(alNPW[PWAI_RLR]-1500)/2
       
      'Single Leg Mode
      elseif bControlMode == SingleLegMode
          LONG[g_palCData][CFG#ISI_SLLegX] := (alNPW[PWAI_RLR]-1500) / 3
          LONG[g_palCData][CFG#ISI_SLLegZ] := -(alNPW[PWAI_RUD]-1500) / 3
          LONG[g_palCData][CFG#ISI_SLLegY] := -(alNPW[PWAI_LLR]-1500)/ 10

      elseif bControlMode == GPPLAYERMODE
          if (LONG[g_palCData][CFG#ISI_GPStart] == 0) and  (alNPW[PWAI_RUD] > 1800) 
              if _fGPEnable
                  LONG[g_palCData][CFG#ISI_GPStart] := 1
              else
                  CFG.MSysSounds (CFG#SSND_BEEP2)
       
      'Calculate walking time delay
      LONG[g_palCData][CFG#ISI_InputTimeDelay] := 128 - (||((alNPW[PWAI_RLR]-1500)/6) #> ||(((alNPW[PWAI_RUD]-1500)/6))) #> ||(((alNPW[PWAI_LLR]-1500)/6))
      LONG[g_palCData][CFG#ISI_SpeedControl] := 0



  else  
      ' Not valid data, let user know that and get back to the Off state...
      if LONG[g_palCData][CFG#ISI_HexOn]
         _cRCFailure++
         if _cRCFailure => MAX_RC_FAILURE
        'Turn off
             LONG[g_palCData][CFG#ISI_BodyPosX] := 0
             LONG[g_palCData][CFG#ISI_BodyPosY] := 0
             LONG[g_palCData][CFG#ISI_BodyPosZ] := 0
             LONG[g_palCData][CFG#ISI_BodyRotX1] := 0
             LONG[g_palCData][CFG#ISI_BodyRotY1] := 0
             LONG[g_palCData][CFG#ISI_BodyRotZ1] := 0
             LONG[g_palCData][CFG#ISI_TravelLengthX] := 0
             LONG[g_palCData][CFG#ISI_TravelLengthZ] := 0
             LONG[g_palCData][CFG#ISI_TravelRotationY] := 0
             LONG[g_palCData][CFG#ISI_SelectedLeg] := 255
                                             
             DBG.str(string("Hex Off",13))
             CFG.MSysSounds(CFG#SSND_HEXOFF)
             LONG[g_palCData][CFG#ISI_HexOn] := 0
                                 

return

pub GPSetState (state)
    case state
        GPERROR:
           CFG.MSysSounds (CFG#SSND_BEEP2)
          LONG[g_palCData][CFG#ISI_GPStart] := 0         ' clear out the state  
        GPSTARTED:
        GPCOMPLETED:
          LONG[g_palCData][CFG#ISI_GPStart] := 0         ' clear out the state  


